/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#include <unistd.h>
#include <RTCPServerSocket.h>

static string localhost("0.0.0.0");

RTCPServerSocket::RTCPServerSocket(Socket &s,
				   RTPSession& rtp_session,
				   const int& mtu):
  RTCPSocket(s, rtp_session, mtu)
{
}

RTCPServerSocket::RTCPServerSocket(const RTCPServerSocket& r):
  RTCPSocket(r)
{
}

RTCPServerSocket::~RTCPServerSocket()
{
}

int RTCPServerSocket::recv_proc()
{
	 int ret=RTCPSocket::recv(_buf, _mtu);
	 if (ret!=_mtu) {
		  syslog(LOG_ERR, "read(): unable to read RTCP packet.");
		  return(-1);
	 }
	 RTCPCommon* h=(RTCPCommon*)_buf;
	 RTCPPacket* hh=(RTCPPacket*)_buf;
	 struct timeval now, lsr, diff;
	 // Currently number of RTCP packets within a compound
	 // packet is hardcoded.
	 // TODO: make this dynamic.
	 const int num_packets=2;
	 u_int8_t* p=NULL;
	 // Lock _rtp_session structure.
	 Guard rtp_session_guard(_rtp_session._mutex);
	 for (int i=0; i<num_packets; ++i) {
		  switch (h->_pt) {
		  case RTCP_RR:
		  {
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received Receiver Report");
				if (gettimeofday(&now, NULL)!=0)
					 syslog(LOG_ERR, "gettimeofday: %s\n", strerror(errno));
#ifdef PRINT
				print_rr(_buf);
#endif // PRINT
				_rtp_session._r._fraction=hh->r.rr._rr[0]._fraction;
				_rtp_session._r._lost=ntohl(hh->r.rr._rr[0]._lost<<8);
				_rtp_session._r._last_seq=ntohl(hh->r.rr._rr[0]._last_seq);
				_rtp_session._r._jitter=ntohl(hh->r.rr._rr[0]._jitter);
				_rtp_session._r._lsr=ntohl(hh->r.rr._rr[0]._lsr);
				_rtp_session._r._dlsr=ntohl(hh->r.rr._rr[0]._dlsr);

				lsr.tv_sec=(_rtp_session._r._lsr>>16);
				lsr.tv_usec=(_rtp_session._r._lsr&0x0000ffff);
				now.tv_sec&=0x0000ffff;                // Lose msbs.
				now.tv_usec&=0xffff0000;        // Lose lsbs.
				timersub(&now, &lsr, &diff);
				_rtp_session._s._rtt=diff.tv_sec+(diff.tv_usec/1e6);
				_rtp_session._s._rtt-=double(_rtp_session._r._dlsr)/65536;
#ifdef CC
				const double cc_thresh=100;
				const double cc_factor=2.0;
				const double cc_add=10000;
				cerr << "Lost: " << _rtp_session._r._lost 
					  << " Cwnd: " << cwnd 
					  << " Frac: " << _rtp_session._r._fraction*1.0/256.0 << endl;
				const double delta=_rtp_session._r._lost-lost;
				lost=_rtp_session._r._lost;
				if (delta>cc_thresh) {
					 cwnd/=cc_factor;
					 cwnd=(cwnd>10)?cwnd:10;
				} else {
					 cwnd*=cc_factor;
					 cwnd=(cwnd>_max_window)?_max_window:cwnd;
				}
#endif
#ifdef PRINT															
				_rtp_session.report(stdout);
#endif // PRINT
				p=(u_int8_t*)hh;
				p+=28;
				h=(RTCPCommon*)p;
				hh=(RTCPPacket*)p;
		  }
		  break;
		  case RTCP_SDES:
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received SDES!\n");
				break;
		  case RTCP_BYE:
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received BYE!\n");
				break;
		  case RTCP_APP:
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received APP!\n");
				break;
		  case RTCP_FB:
		  {
#ifdef PRINT
				print_ack((u_int8_t*)hh);
#endif // PRINT
				// TODO Common processing.
				// hh->r._fb._sender_ssrc;
				// hh->r._fb._src_ssrc;
				// Obtain the Reqest Number from the ack packet.
				u_int16_t rn=ntohs(hh->r._fb._fci[0]._ack._pid);
				// TODO Advanced ACK processing.
				// hh->r._fb._fci[0]._ack._blp;
				// hh->r._fb._fci[0]._ack._r;
				// Clean up rtp_map.
				u_int16_t sn_min=_rtp_session._s._sn_min;
				u_int16_t sn_max=_rtp_session._s._sn_max;
				syslog(LOG_DEBUG, 
						 "PS_RX_RTCP: %s==%d %d %d\n",
						 "<sn_min sn_max rn>",
						 sn_min, sn_max, rn); 
#ifdef RETRANSMISSION
				// These variables for modulo 2^16 arithmetic.
				u_int16_t res1=rn-sn_min;
				u_int16_t res2=sn_max-sn_min;
				// Check that Request Number is in the current
				// window(defined by sn_min and sn_max).
				if (res1<=res2) {
					 u_int16_t i=sn_min;
					 // These variables have changed meaning but
					 // are re-used for module 2^16 arithmetic.
					 // Have to be CAREFUL here. Automatic compiler
					 // casts can really ruin your day/night!
					 res1=i-sn_min;
					 res2=rn-sn_min;
					 // Clean out all packets that have been 
					 // implicitly acknowledged by the Request
					 // Number in the RTCP ack packet.
					 if (RETRANSMISSION) {
						  while (res1<res2) {
								struct iovec* cur=rtp_map[i];
								delete [] (char*)((cur)[0].iov_base);
								delete [] (char*)((cur)[1].iov_base);
								delete [] (cur);
								rtp_map[i++]=0;
								res1=i-sn_min;
						  }
					 }
					 // Update sn_min to point to the first
					 // unacknowledged packet stored in rtp_map.
					 _rtp_session._s._sn_min=rn;
				} else {
					 syslog(LOG_ERR, "PS_RX_RTCP: %s==%d %d %d",
							  "Received out of range <rn sn_min sn_max>",
							  rn, sn_min, sn_max);
				}
#else
				_rtp_session._s._sn_min=rn;
#endif
				// TODO NACK Processing.
				// hh->r._fb._fci[0]._sender_ssrc;
				// hh->r._fb._fci[0]._src_ssrc;
				// hh->r._fb._fci[0]._nack._pid;
				// hh->r._fb._fci[0]._nack._blp;
		  }
		  break;
		  case RTCP_PSFB:
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received PSFB!");
				break;
		  default:                    
				syslog(LOG_NOTICE, "PS_RX_RTCP: Received unknown RTCP packet!");
				break;
		  }
	 }
	 return(0);
}

int RTCPServerSocket::send_proc()
{
	 _rtcp_duration.stop();
	 double delta=8.0*(_rtp_session._s._osent-_prior_osent);
	 double throughput = delta/_rtcp_duration.elapsed();
	 _prior_osent=_rtp_session._s._osent;
	 cerr << "Throughput: " << throughput << " bps" << endl;				
	 
	 unsigned int count=1;
	 struct timeval ntp_ts;
	 if (gettimeofday(&ntp_ts, NULL)!=0)
		  syslog(LOG_ERR, "RTCPServerSocket: Error reading NTP timestamp\n");
	 syslog(LOG_DEBUG, "About to send SR");
	 unsigned int ntp_sec=ntp_ts.tv_sec;
	 unsigned int ntp_frac=ntp_ts.tv_usec;
	 // Lock RTP session structure.
	 // Automatically unlocks when Guard object goes out of scope.
	 Guard rtp_session_guard(_rtp_session._mutex);
	 build_sr(_buf, 
				 _mtu,
				 count,
				 _rtp_session._ssrc,
				 ntp_sec,
				 ntp_frac,
				 _rtp_session._s._rtp_ts,
				 _rtp_session._s._psent,
				 _rtp_session._s._osent);
#ifdef PRINT
	 print_sr(_buf);
#endif // PRINT
	 // Transmit RTCP packet.
	 int ret=0;
	 
	 ret=RTCPSocket::send(_buf, _mtu);
	 if (ret!=_mtu) {
		  syslog(LOG_ERR, "send(): error: %s", strerror(errno));
		  return(-1);
	 }
	 // Update average RTCP packet size.
	 _rtp_session._r._avg_rtcp_size = (1/16)*(ret+8+20)+                                                                                                
		  (15/16) * _rtp_session._r._avg_rtcp_size;
	 _rtp_session.update_rate();
	 syslog(LOG_DEBUG, "RTCPServeerSocket::send(): bytes_sent==%d\n", ret);
	 _rtcp_duration.start();
	 return(_mtu);
}


