/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef RTPTHREAD_H
#define RTPTHREAD_H

#include <common.h>
#include <Socket.h>
#include <RTPSocket.h>
#include <TCPSocket.h>
#include <string>
#include <iostream>
#include <Thread.h>

/** RTPThread class
  * @doc This class encapsulates a UDP stream and implements the VLBI Real-time
  * protocol on top of it.
  */

class RTPThread: public Thread
{
	/** Data transmission.
	  * All data is transmitted in one direction along this socket.
	  */
  RTPSocket& _rtp_socket;
	 /** State.
	  * RTPSession reference that stores RTP session state.
	  */
  RTPSession& _rtp_session;
	/** MTU size.
	  * MTU size for data transmission, receptions.
	  */
  int _mtu;
	/** Buffer.
	 */
  u_int8_t* _buf;
	/** Thread callback.
	  * This function is called when the base class method "thread_create()" is
	  * called. This results in thread_func() being executed in it's own
	  * thread. Depending on the mode, either recv_proc() or send_proc() will
	  * be called by thread_func(). This means that recv_proc() or send_proc()
	  * will be running in their own thread of execution.
	  */
  void* thread_func();
	/** Thread start.
	 * This function starts the separate thread of execution.
	 */
  void start();
public:
	/** Default constructor.
	  * This calls the base constructor to create a UDP socket and
	  * initialize a thread to service it.
	  * @param rtp_socket a reference to an RTPSocket that will be used to transfer
	  * data in RTP format.
	  * @param rtp_session a reference to an RTPSession that will be used to
	  * store the protocol state.
	  * @param mtu size of packet payload in bits.
	  * store the protocol state.
	  */
  RTPThread(RTPSocket& rtp_socket,
	    RTPSession& rtp_session,
	    const int& mtu);
	/** Copy constructor.
	  * Copies values of other RTPThread into this object.
	  * @param mode The mode for this socket (either send or receive).
	  * @param s RTPThread to copy.
	  */
  RTPThread(const RTPThread &s);
	/** Destructor. */
  virtual ~RTPThread();
	/** Assigment operator.
	  * Copies supplied socket fields into this object.
	  * @param s thread to copy.
	  */
  RTPThread& operator=(const RTPThread& s);
};

ostream& operator<<(ostream& os, const RTPThread& s);

#endif

