/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2004 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifdef UDT_SUPPORTED

#include <UDT2Socket.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <iostream>
#include <errno.h>
#include <syslog.h>

#define INVALID_UDTSOCK -1

using namespace UDT;

UDT2Socket::UDT2Socket():
		  Socket()
{
	 // NO socket created.
}

UDT2Socket::UDT2Socket(const int& m):
		  Socket(m)
{
	 _sockd=UDT::socket(PF_INET, SOCK_STREAM, 0);
	 if (_sockd==INVALID_UDTSOCK) 
		  syslog(LOG_ERR, "UDT2Socket::UDT2Socket(): unable to create socket.");
	 set_so_reuseaddr(1);
}

UDT2Socket::UDT2Socket(const int& s, const int& m):
		  Socket(s, m)
{
	 set_so_reuseaddr(1);
}

UDT2Socket::UDT2Socket(const UDT2Socket &s):
		  Socket(s)
{
	 set_so_reuseaddr(1);
}

UDT2Socket::~UDT2Socket()
{
    // This is an important feature of the UDT2Socket class. Especially
    // for copying of UDT2Sockets. UDT2Sockets do *not* close their
	 // sockets when they are destroyed.
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): _bytes_sent==%lld", _bytes_sent);
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): _bytes_rcvd==%lld", _bytes_rcvd);
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): _send_calls==%lld", _send_calls);
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): _recv_calls==%lld", _recv_calls);
	 _age.stop();
	 double duration=_age.elapsed();
	 double rcv_rate=_bytes_rcvd/duration*8.0/1000000;
	 double send_rate=_bytes_sent/duration*8.0/1000000;
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): duration==%f", duration);
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): rcv_rate==%f", rcv_rate);
	 syslog(LOG_ERR, "UDT2Socket::~UDT2Socket(): send_rate==%f", send_rate);
}

UDT2Socket& UDT2Socket::operator=(const UDT2Socket& s)
{
    _sockd=s._sockd;
	 _mtu=s._mtu;
	 set_so_reuseaddr(1);
    return(*this);
}

void UDT2Socket::set_sockd(const int& s)
{
    if (_sockd!=INVALID_UDTSOCK)
        UDT::close(_sockd);
    _sockd=s;
}

void UDT2Socket::set_mtu(const int& m)
{
	 _mtu=m;
}

int UDT2Socket::bind(const string& ip, const int& port)
{
    char* p=(char*)ip.data();
    struct sockaddr_in addr;
    int ret=0;
    if (ip.size()<0) {
        syslog(LOG_ERR, "UDT2Socket::bind(): zero length IP address  string.");
    }      
    ret=::inet_aton(p, &addr.sin_addr);
#ifndef LINUX
    addr.sin_len=sizeof(addr);
#endif // LINUX
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    if (ret==0) {
        syslog(LOG_ERR, "UDT2Socket::bind(): Invalid IP address string.");
    }
    if (_sockd==-1) {
        syslog(LOG_ERR, "UDT2Socket::bind(): Invalid sock descriptor.");
    }
    ret=UDT::bind(_sockd, (struct sockaddr*)&addr, sizeof(addr));
    if (ret==-1)
		  syslog(LOG_ERR, "UDT2Socket::bind() error: %s", strerror(errno));
	 return(ret);
}

int UDT2Socket::listen()
{
    if (_sockd==-1)
		  syslog(LOG_WARNING, "UDT2Socket: Invalid sock descriptor");
    int ret=UDT::listen(_sockd, 10);
	 if (ret==-1)
		  syslog(LOG_ERR, "UDT2Socket: Listen error.");
	 return(ret);
}

int UDT2Socket::accept() 
{
	 if (_sockd==INVALID_UDTSOCK)
		  syslog(LOG_ERR, "UDT2Socket::accept(): Invalid sock descriptor.");
	 struct sockaddr addr;
	 socklen_t addrlen=sizeof(addr);
	 int ret=UDT::accept(_sockd, &addr, &addrlen);
	 if (ret==INVALID_UDTSOCK)
		  syslog(LOG_ERR, "UDT2Socket: Accept error: %s", strerror(errno));
	 return(ret);
}

int UDT2Socket::connect(const string& ip, const int& port)
{
    struct sockaddr_in addr;
    int ret=0;
    if (ip.size()<=0) {
        syslog(LOG_ERR, "UDT2Socket::connect(): Zero length IP address string.");
    }
    ret=::inet_aton((char*)ip.data(), &addr.sin_addr);
    if (ret==0) {
        syslog(LOG_ERR, "Invalid IP address string");
    }
    addr.sin_port=htons(port);
#ifndef LINUX
    addr.sin_len=sizeof(addr);
#endif // LINUX
    addr.sin_family=AF_INET;
    ret=UDT::connect(_sockd, (struct sockaddr*)&addr, sizeof(addr));
	 if (ret==-1)
		  syslog(LOG_ERR, "UDT2Socket: connect() error: %s", strerror(errno));
	 return(ret);
}

int UDT2Socket::close()
{
    return(UDT::close(_sockd));
}

int UDT2Socket::shutdown(const int& howto)
{
    if (_sockd==INVALID_UDTSOCK) {
        syslog(LOG_ERR, "UDT2Socket::shutdown(): Attempt to close unallocated socket");
    }
    return(UDT::shutdown(_sockd, howto));
}

ostream& operator<<(ostream& os, const UDT2Socket& s)
{
	 os << "UDT2Socket dump\n";
    os << "    _sockd:         " << s.get_sockd() << endl;
    os << "    _bytes_sent:    " << s.get_bytes_sent() << endl;
    os << "    _bytes_rcvd:    " << s.get_bytes_rcvd() << endl;
    os << "    _send_calls:    " << s.get_send_calls() << endl;
    os << "    _recv_calls:    " << s.get_recv_calls() << endl;
	 os << "-UDT2Socket dump\n";
	 os << "UDT2Socket dump\n";
    return(os);
}


#endif   // UDT_SUPPORTED


