/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2004 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */


#include <Socket.h>
#include "UnixSocket.h"
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <iostream>
#include <errno.h>
#include <syslog.h>


UnixSocket::UnixSocket():
		  Socket()
{
}

UnixSocket::UnixSocket(const int& m):
		  Socket(m)

{
	 _sockd=socket(PF_LOCAL, SOCK_STREAM, 0);
	 set_so_reuseaddr(1);
}

UnixSocket::UnixSocket(const int& s, const int& m):
		  Socket(s, m)
{
	 _sockd=socket(PF_LOCAL, SOCK_STREAM, 0);
	 set_so_reuseaddr(1);
}

UnixSocket::UnixSocket(const UnixSocket &s):
		  Socket(s)
{
	 set_so_reuseaddr(1);
}

UnixSocket::~UnixSocket()
{
    // This is an important feature of the UnixSocket class. Especially
    // for copying of UnixSockets. UnixSockets do *not* close their
	 // sockets when they are destroyed.
}

UnixSocket& UnixSocket::operator=(const UnixSocket& s)
{
    _sockd=s._sockd;
	 _mtu=s._mtu;
	 set_so_reuseaddr(1);
    return(*this);
}

void UnixSocket::set_sockd(const int& s)
{
    if (_sockd!=-1)
        ::close(_sockd);
    _sockd=s;
}

int UnixSocket::bind(const string& path, const int& port)
{
    struct sockaddr_un addr;
    int ret=0;
    addr.sun_family=AF_LOCAL;
    strncpy(addr.sun_path, path.c_str(), 104);
    if (_sockd==-1) {
        syslog(LOG_ERR, "UnixSocket::bind(): Invalid sock descriptor.");
    }
    ret=::bind(_sockd, (struct sockaddr*)&addr, sizeof(struct sockaddr_un));
    if (ret==-1)
		  syslog(LOG_ERR, "Bind error: %s", strerror(errno));
    // FIX.
    struct sockaddr_un addr2;
    socklen_t len = sizeof(addr2);
    getsockname(_sockd, (struct sockaddr*)&addr2, &len);
    syslog(LOG_DEBUG, "UnixSocket::bind(): Bound name: %s, length: %d",
			  addr2.sun_path, len);
	 return(ret);
}

int UnixSocket::listen()
{
    if (_sockd==-1)
		  syslog(LOG_WARNING, "UnixSocket::listen(): Invalid sock descriptor");
    int ret=::listen(_sockd, 10);
	 if (ret==-1)
		  syslog(LOG_ERR, "UnixSocket::listen(): error: %s", strerror(errno));
	 return(ret);
}

int UnixSocket::accept()
{
    if (_sockd==-1) {
        syslog(LOG_ERR, "UnixSocket::accept(): Invalid sock descriptor.");
    }
    struct sockaddr addr;
    socklen_t addrlen=sizeof(addr);
    int ret=::accept(_sockd, &addr, &addrlen);
	 if (ret==-1)
		  syslog(LOG_ERR, "UnixSocket::accept(): error: %s", strerror(errno));
    return(ret);
}

int UnixSocket::connect(const string& path)
{
    struct sockaddr_un addr;
    int ret=0;
    addr.sun_family=AF_LOCAL;
    strncpy(addr.sun_path, path.c_str(), 104);
    ret=::connect(_sockd, (struct sockaddr*)&addr, sizeof(addr));
	 if (ret==-1)
		  syslog(LOG_ERR, "UnixSocket::connect() error: %s", strerror(errno));
	 return(ret);
}

int UnixSocket::close()
{
    return(Socket::close());
}

int UnixSocket::shutdown(int howto)
{
    if (_sockd==-1) {
        syslog(LOG_ERR, "UnixSocket::shutdown(): attempt to close unallocated socket");
    }
    return(::shutdown(_sockd, howto));
}

ostream& operator<<(ostream& os, const UnixSocket& s)
{
	 os << "UnixSocket dump\n";
    os << "    _sockd:         " << s.get_sockd() << endl;
    os << "    _bytes_sent:    " << s.get_bytes_sent() << endl;
    os << "    _bytes_rcvd:    " << s.get_bytes_rcvd() << endl;
    os << "    _send_calls:    " << s.get_send_calls() << endl;
    os << "    _recv_calls:    " << s.get_recv_calls() << endl;
	 os << "-UnixSocket dump\n";
	 os << "UnixSocket dump\n";
    return(os);
}



